> 英語以外の言語で書かれたドキュメントでは、文章が不自然になることがあります。ドキュメントの改善にご協力いただける方は、[Pull request](https://github.com/cunarist/rust-in-flutter/pulls)をお願いいたします。いつもご協力に感謝いたします。

# 🆎 Rust-In-Flutter

Flutter アプリを驚くほど高速化するために、簡単に Rust を統合しましょう！

![プレビュー](https://github.com/cunarist/rust-in-flutter/assets/66480156/be85cf04-2240-497f-8d0d-803c40536d8e)

この軽量なフレームワークは、使いやすさ、将来の拡張性、優れたパフォーマンスを考慮して設計されており、裏側の複雑な部分をすべて処理します。このパッケージを Flutter プロジェクトに追加するだけで、Rust のコードを簡単に書くことができます！

## 利点

- Rust の統合：任意のライブラリクレートの使用が可能
- 既存の Rust クレートをそのまま使用可能
- CMake、Gradle、Podfile などの機密ビルドファイルを扱う必要なし
- 開発中の複雑なコード生成なし
- 多くの労力をかけずに無制限の RESTful API エンドポイントの定義
- Dart からの簡単なリクエストと Rust からの非同期相互作用
- Rust から Dart へのストリーミング
- Dart のホットリスタート時に Rust ロジックを再起動
- 最小限のオーバーヘッド
- ネイティブデータ送信時のメモリコピーなし

## プラットフォームサポート

このパッケージによって、難しいビルド設定は自動的に処理されます。Flutter プロジェクトのファイルは影響を受けないことに注意してください。

- ✅ Linux：テスト済みサポート
- ✅ Android：テスト済みサポート
- ✅ Windows：テスト済みサポート
- ✅ macOS：テスト済みサポート
- ✅ iOS：テスト済みサポート
- ⏸️ Web：現時点ではサポートされていません [検討中](https://github.com/cunarist/rust-in-flutter/issues/34)

> 提案がある場合やバグを報告したい場合は、[issue](https://github.com/cunarist/rust-in-flutter/issues)または[pull request](https://github.com/cunarist/rust-in-flutter/pulls)として残してください。できる限り迅速に対応いたします。

## Rust の利用理由

Dart は GUI アプリ向けの素晴らしいオブジェクト指向のモダンな言語ですが、ネイティブではないガベージコレクションの言語であるため、そのパフォーマンスが要件を満たさない場合があります。そこで Rust が登場します。Rust のパフォーマンスは、Dart よりもおよそ[2~40 倍高速](https://programming-language-benchmarks.vercel.app/dart-vs-rust)であるとされています。さらに、複数のスレッドを利用する能力も備えています。

Rust は、Stack Overflow 上で[最も愛されているプログラミング言語](https://survey.stackoverflow.co/2022#section-most-loved-dreaded-and-wanted-programming-scripting-and-markup-languages)と報告されており、そのゼロキャスト抽象哲学による高い生産性とネイティブパフォーマンスが提供されています。

もちろん、C や C++もありますが、多くの開発者が[Rust が将来的に C++を置き換える](https://dev.to/roy8/why-rust-will-replace-c-in-the-future-49op)と信じています。Rust が C++よりも人気がある理由はいくつかあります。まず第一に、Rust ははるかにシンプルで学びやすいです。これは、シンプルさと安全性に焦点を当てた設計によるものです。さらに、Rust は多くの場面で C++よりも優れたパフォーマンスを提供します。また、より活発なコミュニティと優れたツールサポートを持っています。

Rust の詳細は[公式の書籍](https://doc.rust-lang.org/book/foreword.html)で学び、理解することができます。

# 👜 コンポーネントのインストール

このセクションでは、すでに[Flutter SDK](https://docs.flutter.dev/get-started/install)をシステムにインストールし、`flutter create`コマンドで Flutter プロジェクトを作成したと仮定しています。まだ Flutter プロジェクトを作成していない場合は、[この素晴らしいチュートリアル](https://docs.flutter.dev/get-started/codelab)に従って作成してください。

まず、このパッケージを Flutter プロジェクトに追加します。

```bash
flutter pub add rust_in_flutter
```

そして、Rust ツールチェーンをインストールしてください。公式 Rust ドキュメントを参照してください：[公式 Rust ドキュメント](https://doc.rust-lang.org/book/ch01-01-installation.html)。

最後に、システムがコンパイルに対して準備ができていることを確認してください。各インストールステップの後でシステムの状態を確認するために以下のコマンドを繰り返すことができます。出力に問題がない場合は、準備が整っています！

```bash
rustc --version
flutter doctor
```

## ビルドツールバージョンの問題

- Android アプリの場合、[この問題](https://github.com/rust-lang/rust/pull/85806)のために Rust 1.68 以上を使用する必要があります。
- Android アプリの場合、`./android/app/build.gradle`内の変数`ndkVersion`の設定が必要ですが、Flutter SDK 3.7 以前で Flutter プロジェクトを作成した場合には、これが欠落している可能性があります。[このディスカッション](https://github.com/cunarist/rust-in-flutter/discussions/60)を参照して、この問題を解決してください。

> Rust を使用してさまざまなビルドターゲットを利用する際には、さまざまな問題が発生することがあります。何か問題が発生した場合は、[ディスカッションページ](https://github.com/cunarist/rust-in-flutter/discussions)を訪れ、アシスタンスのための Q&A スレッドを開いてください。

# 👜 テンプレートの適用

単純に、Flutter プロジェクトのディレクトリで以下のコマンドをコマンドラインで実行してください。

```bash
dart run rust_in_flutter:apply_template
```

このコマンドを実行した後、新しいファイルとフォルダが生成され、スターター Rust テンプレートとなります。

```diff
    my_flutter_project/
    ├── android/
    ├── ios/
    ├── lib/
*   │   ├── main.dart
    │   └── ...
    ├── linux/
+   ├── native/
+   │   ├── hub/
+   │   │   ├── src/
+   │   │   └── Cargo.toml
+   │   ├── sample_crate/
+   │   │   ├── src/
+   │   │   └── Cargo.toml
+   │   └── README.md
    ├── web/
    ├── windows/
*   ├── .gitignore
+   ├── Cargo.toml
*   ├── pubspec.yaml
    └── ...
```

`./native/README.md`ファイルを最初に読むことを忘れないでください。コードにはさまざまなコメントが記述されており、構造を理解するのに役立ちます。また、本番環境では`sample_crate`を削除することを検討するかもしれません。すでに使用したい Rust クレートがある場合は、それを`./native`内に配置し、`hub`クレートの依存として設定してください。

これで、`./native/hub/src/lib.rs`に移動して、Rust のコードを書き始めることができます！

# 🧱 コードの書き方

## Dart からのリクエスト、Rust からのレスポンス

アプリが大きくなるにつれて、新しい Rust API エンドポイントを定義する必要があります。

Dart から Rust に配列と文字列を送信し、それに対していくつかの計算を行う新しいボタンを作成したいとします。リクエストを送信してレスポンスを待つ方法を理解するために、以下の手順に従うことができます。

[デフォルトの例](https://github.com/cunarist/rust-in-flutter/tree/main/example)を起点にして、Dart でユーザーの入力を受け付けるボタンウィジェットを作成します。

```diff
  // lib/main.dart
  ...
  child: Column(
    mainAxisAlignment: MainAxisAlignment.center,
    children: [
+     ElevatedButton(
+       onPressed: () async {},
+       child: Text("Request to Rust"),
+     ),
  ...
```

`onPressed`関数は Rust にリクエストを送信する必要があります。まずは`RustRequest`オブジェクトを作成しましょう。

```diff
  // lib/main.dart
  ...
  import 'package:rust_in_flutter/rust_in_flutter.dart';
  ...
  ElevatedButton(
+   onPressed: () async {
+     final rustRequest = RustRequest(
+       address: 'myCategory.someData',
+       operation: RustOperation.Read,
+       bytes: serialize(
+         {
+           'input_numbers': [3, 4, 5],
+           'input_string': 'Zero-cost abstraction',
+         },
+       ),
+     );
+   },
    child: Text("Request to Rust"),
  ),
  ...
```

`address`はアプリのデザインに合った任意の文字列で、ドットで層になっているキャメルケースの文字列として表現されます。`operation`は、このシステムが RESTful API の定義に従っているため、作成、読み取り、更新、削除のいずれかになります。そして、`bytes`は単純なバイト配列であり、通常は[MessagePack](https://msgpack.org/)のシリアル化によって作成されます。

それでは、このリクエストを Rust に送信します。これを行うのは`requestToRust`関数で、`RustResponse`オブジェクトを返します。

```diff
  // lib/main.dart
  ...
  import 'package:rust_in_flutter/rust_in_flutter.dart';
  ...
  ElevatedButton(
    onPressed: () async {
      final rustRequest = RustRequest(
        address: 'myCategory.someData',
        operation: RustOperation.Read,
        // Dartのオブジェクトを生のバイト配列に変換するために、
        // `msgpack_dart.dart`が提供する`serialize`関数を使います。
        bytes: serialize(
          {
            'input_numbers': [3, 4, 5],
            'input_string': 'Zero-cost abstraction',
          },
        ),
      );
+     final rustResponse = await requestToRust(rustRequest);
    },
    child: Text("Request to Rust"),
  ),
    ...
```

したがって、新しい API アドレスは`myCategory.someData`です。Rust のリクエストハンドラ関数がこれを受け入れることを確認してください。

```diff
    // native/hub/src/with_request.rs
    ...
    use crate::bridge::api::RustResponse;
    use crate::sample_functions;
    ...
    let rust_response = if layered.is_empty() {
        RustResponse::default()
    } else if layered[0] == "basicCategory" {
        if layered.len() == 1 {
            RustResponse::default()
        } else if layered[1] == "counterNumber" {
            sample_functions::calculate_something(rust_request).await
        } else {
            RustResponse::default()
        }
+   } else if layered[0] == "myCategory" {
+       if layered.len() == 1 {
+           RustResponse::default()
+       } else if layered[1] == "someData" {
+           sample_functions::some_data(rust_request).await
+       } else {
+           RustResponse::default()
+       }
    } else {
        RustResponse::default()
    };
    ...
```

これが新しいエンドポイントの Rust 関数である`sample_functions::some_data`です。このシンプルな API エンドポイントでは、配列の各要素に 1 を加算し、文字列のすべての文字を大文字に変換してそれらを返します。メッセージスキーマは、操作のタイプによって異なるため、マッチステートメントで定義されています。

```diff
    // native/hub/src/sample_functions.rs
    ...
    use crate::bridge::api::RustOperation;
    use crate::bridge::api::RustRequest;
    use crate::bridge::api::RustResponse;
    use rmp_serde::from_slice;
    use rmp_serde::to_vec_named;
    use serde::Deserialize;
    use serde::Serialize;
    ...
+   pub async fn some_data(rust_request: RustRequest) -> RustResponse {
+       match rust_request.operation {
+           RustOperation::Create => RustResponse::default(),
+           RustOperation::Read => {
+               #[allow(dead_code)]
+               #[derive(Deserialize)]
+               struct RustRequestSchema {
+                   input_numbers: Vec<i8>,
+                   input_string: String,
+               }
+               let slice = rust_request.bytes.as_slice();
+               let received: RustRequestSchema = from_slice(slice).unwrap();
+
+               let new_numbers = received.input_numbers.into_iter().map(|x| x + 1).collect();
+               let new_string = received.input_string.to_uppercase();
+
+               #[derive(Serialize)]
+               struct RustResponseSchema {
+                   output_numbers: Vec<i8>,
+                   output_string: String,
+               }
+               RustResponse {
+                   successful: true,
+                   bytes: to_vec_named(&RustResponseSchema {
+                       output_numbers: new_numbers,
+                       output_string: new_string,
+                   })
+                   .unwrap(),
+               }
+           }
+           RustOperation::Update => RustResponse::default(),
+           RustOperation::Delete => RustResponse::default(),
+       }
+   }
    ...
```

## Rust から Dart へのストリーミング

Rust から Dart に毎秒増加する数値を送信したいとします。この場合、Dart が繰り返しリクエストを送信するのは非効率です。ここでストリーミングが必要になります。

[デフォルトの例](https://github.com/cunarist/rust-in-flutter/tree/main/example)を起点にします。Rust で非同期関数を生成します。

```diff
    // native/hub/src/lib.rs
    ...
    use tokio::task::spawn;
    ...
    mod sample_functions;
    ...
    spawn(sample_functions::keep_drawing_mandelbrot());
+   spawn(sample_functions::keep_sending_numbers());
    while let Some(request_unique) = request_receiver.recv().await {
    ...
```

永遠に実行される非同期 Rust 関数を定義し、毎秒数値を Dart に送信します。

```diff
    // native/hub/src/sample_functions.rs
    ...
    use crate::bridge::api::RustSignal;
    use crate::bridge::send_rust_signal;
    ...
    use rmp_serde::to_vec_named;
    ...
    use serde::Serialize;
    ...
+   pub async fn keep_sending_numbers() {
+       let mut current_number: i32 = 1;
+       loop {
+           tokio::time::sleep(std::time::Duration::from_secs(1)).await;
+
+           #[derive(Serialize)]
+           struct RustSignalSchema {
+               current_number: i32,
+           }
+           let rust_signal = RustSignal {
+               address: String::from("sampleCategory.mandelbrot"),
+               bytes: to_vec_named(&RustSignalSchema {
+                   current_number: current_number,
+               })
+               .unwrap(),
+           };
+           send_rust_signal(rust_signal);
+           current_number += 1;
+       }
+   }
    ...
```

最後に、`StreamBuilder`を使用して Dart で信号を受信し、`where`メソッドでアドレスでフィルタリングし、ウィジェットを再構築します。

```diff
  // lib/main.dart
  ...
  import 'package:msgpack_dart/msgpack_dart.dart';
  import 'package:rust_in_flutter/rust_in_flutter.dart';
  ...
  children: [
+   StreamBuilder<RustSignal>(
+     stream: rustBroadcaster.stream.where((rustSignal) {
+       return rustSignal.address == "myCategory.increasingNumbers";
+     }),
+     builder: (context, snapshot) {
+       final received = snapshot.data;
+       if (received == null) {
+         return Text("Nothing received yet");
+       } else {
+         final signal = deserialize(received.bytes) as Map;
+         final currentNumber = signal["current_number"] as int;
+         return Text(currentNumber.toString());
+       }
+     },
+   ),
```

以上の内容です。

# ✋ ヒント

理想的には、**Flutter**はクロスプラットフォームのユーザーインターフェースを担当し、**Rust**はビジネスロジックを処理します。フロントエンドとバックエンドは完全に分離されるため、Dart と Rust のコードは互いに独立しています。これらの 2 つの世界はチャンネルとストリームを介して通信します。

[MessagePack](https://msgpack.org/)を使用して、Dart と Rust 間で送信されるメッセージを直接 Rust テンプレートで提供される形式でシリアライズします（他の理由がない限り）。MessagePack は、JSON に似た入れ子構造のバイナリ形式であり、高速かつコンパクトです。MessagePack は JSON よりも多くの種類の内部データをサポートしています。これには、バイナリデータも含まれます。詳細なタイプシステムの仕様については、[このリンク](https://github.com/msgpack/msgpack/blob/master/spec.md#type-system)を参照してください。

Dart と Rust の間で送信されるデータは基本的にバイト配列であり、Dart では`Uint8List`、Rust では`Vec<u8>`として表現されます。MessagePack のシリアライズを使用することをお勧めしますが、高解像度の画像やファイルデータなど、任意のバイトデータを送信することができます。また、必要な場合は空のバイト配列を送信することもできます。

Rust-In-Flutter のすべてのビルド設定により、Rust クレートからコンパイルされたすべてのライブラリファイルが適切に最終ビルドに含まれ、配布の準備が整います。したがって、ライブラリファイルのバンドルについて心配する必要はありません。

その他の詳細なお知らせや情報は、[ディスカッションページ](https://github.com/cunarist/rust-in-flutter/discussions)で共有されています。追加のガイドを読むや質問をするには、このページを訪れてください。

# ☕ サポートをお願いします

😉 Rust-In-Flutter の機能を活用し、役立つと感じている場合は、このプロジェクトのサポートを検討してみてください。寄付いただいた方々の寛大な支援により、Rust-In-Flutter の保守と開発が行われ、継続的な改善と成長が実現されます。

もし気に入っていただけたら、[コーヒーをおごっていただけると嬉しいです](https://www.buymeacoffee.com/cunarist)。
